/**********************************************************************
  string buffer function
  $Id: strbuf.c,v 1.1 2003/07/06 01:37:05 tane Exp $
**********************************************************************/
#include <stdlib.h>
#include <stdio.h> 
#include <strings.h>
#include "strbuf.h"

struct STRBUF* STR_alloc()
{
  struct STRBUF *sbp ;

  sbp=malloc( sizeof( struct STRBUF ) ) ;
  if( sbp==NULL ) return(NULL) ;

  sbp->elm =NULL ;
  sbp->size=0 ;
  sbp->ptr =0 ;

  return( sbp ) ;
}


int STR_free( struct STRBUF *sbp )
{
  struct STRELM *cur ;
  struct STRELM *next ;

  if( sbp==NULL ) return( -1 ) ;

  cur=sbp->elm ;
  if( cur!=NULL ){
    while( -1 ){
      if( cur->str!=NULL ){
	free(cur->str) ;
      }
      next=cur->next ;
      free(cur) ;
      if( next==NULL ) break ;
      cur=next ;
    }
  }

  free( sbp ) ;

  return( 0 ) ;
}


unsigned char* STR_addlist( struct STRBUF *sbp, unsigned char *str )
{
  struct STRELM *cur ;
  struct STRELM *new ;
  int len ;

  if( sbp==NULL ) return(NULL) ;
  
  if( sbp->elm==NULL ){
    new=malloc( sizeof(struct STRELM) ) ;
    if( new==NULL ) return(NULL) ;
    new->next=NULL ;
    new->prev=NULL ;
    len=strlen(str) ;
    new->str=malloc( len+1 ) ;
    if( new->str==NULL ){
      free(new) ;
      return(NULL) ;
    }
    sbp->elm=new ;
  }else{
    cur=sbp->elm ;
    while( -1 ){
      if( cur->next==NULL ) break ;
      cur=cur->next ;
    }
    
    new=malloc( sizeof(struct STRELM) ) ;
    if( new==NULL ) return(NULL) ;
    new->prev=cur ;
    new->next=NULL ;
    
    len=strlen(str) ;
    new->str=malloc( len+1 ) ;
    if( new->str==NULL ){
      free(new) ;
      return(NULL) ;
    }
    cur->next=new ;
  }
  strcpy( new->str, str ) ;
  sbp->size++ ;
  return(new->str) ;
}


unsigned char* STR_addtoplist( struct STRBUF *sbp, unsigned char *str )
{
  struct STRELM *cur ;
  struct STRELM *new ;
  int len ;

  if( sbp==NULL ) return(NULL) ;
  
  if( sbp->elm==NULL ){
    new=malloc( sizeof(struct STRELM) ) ;
    if( new==NULL ) return(NULL) ;
    new->next=NULL ;
    new->prev=NULL ;
    len=strlen(str) ;
    new->str=malloc( len+1 ) ;
    if( new->str==NULL ){
      free(new) ;
      return(NULL) ;
    }
    sbp->elm=new ;
  }else{
    cur=sbp->elm ;
    new=malloc( sizeof(struct STRELM) ) ;
    if( new==NULL ) return(NULL) ;
    new->prev=NULL ;
    new->next=cur ;
    
    len=strlen(str) ;
    new->str=malloc( len+1 ) ;
    if( new->str==NULL ){
      free(new) ;
      return(NULL) ;
    }
    sbp->elm=new ;
  }
  strcpy( new->str, str ) ;
  sbp->size++ ;
  return(new->str) ;
}


int STR_dellist( struct STRBUF *sbp, int n )
{
  struct STRELM *cur ;
  struct STRELM *cur_next ;
  struct STRELM *cur_prev ;
  int i ;
  int is_top ;

  if( sbp==NULL ) return( -1 ) ;

  if( n<0 || n>=sbp->size ){
    return( -1 ) ;
  }

  cur=sbp->elm ;
  if( cur==NULL ) return( -1 ) ;

  is_top=0 ;
  if( cur->next==NULL && cur->prev==NULL ){
    is_top=1 ;
  }

  for( i=0 ; i<n ; i++ ){
    cur=cur->next ;
  }

  cur_next=cur->next ;
  cur_prev=cur->prev ;

  if( cur_next!=NULL ) cur_next->prev=cur_prev ;
  if( cur_prev!=NULL ) cur_prev->next=cur_next ;

  if( cur->str!=NULL ){
    free(cur->str) ;
  }
  free(cur) ;
  
  if( n==0 ){
    if( is_top==1 ){
      sbp->elm=NULL ;
    }else{
      sbp->elm=cur_next ;
    }
  }

  sbp->size-- ;
  return(0) ;
}


unsigned char* STR_refstr( struct STRBUF *sbp, int n )
{
  struct STRELM *cur ;
  int i ;
  
  if( sbp==NULL ) return( NULL ) ;

  if( n<0 || n>=sbp->size ) return( NULL ) ;

  cur=sbp->elm ;
  if( cur==NULL ) return( NULL ) ;
  
  for( i=0 ; i<n ; i++ ){
    cur=cur->next ;
  }

  return( cur->str ) ;
}


unsigned char* STR_replace( struct STRBUF *sbp, int n, unsigned char *str )
{
  struct STRELM *cur ;
  unsigned char *buf ;
  int i ;
  
  if( sbp==NULL ) return(NULL) ;

  if( n<0 || n>=sbp->size ) return(NULL) ;

  buf=malloc( strlen(str)+1 ) ;
  if( buf==NULL ) return(NULL) ;
  
  cur=sbp->elm ;
  if( cur==NULL ) return(NULL) ;
  
  for( i=0 ; i<n ; i++ ){
    cur=cur->next ;
  }

  if( cur->str!=NULL ){
    free( cur->str ) ;
  }
  cur->str=buf ;

  strcpy( cur->str, str ) ;
  
  return( cur->str ) ;
}


int STR_buflen( struct STRBUF *sbp )
{
  
  if( sbp==NULL ) return( -1 ) ;

  return( sbp->size ) ;
}

#ifdef STRBUF_TEST

int main(int argc, char *argv[])
{

  struct STRBUF *sbp ;
  int i ;
    
  sbp=STR_alloc() ;
  if( sbp==NULL ) exit(-1) ;
  
  printf("buffer length = %d\n",STR_buflen(sbp)) ;
  for( i=0 ; i<argc ; i++ ){
    printf("%s\n",STR_addlist(sbp,argv[i])) ;
    printf("buffer length = %d\n",STR_buflen(sbp)) ;
  }

  for( i=0 ; i<argc ; i++ ){
    printf("%s\n",STR_refstr(sbp,i)) ;
  }
  
  for( i=0 ; i<argc ; i++ ){
    printf("%s\n",STR_replace(sbp,i,"aaaaaaa")) ;
  }

  for( i=0 ; i<argc ; i++ ){
    printf("%s\n",STR_refstr(sbp,i)) ;
  }

  for( i=0 ; i<argc ; i++ ){
    printf("dellist return=%d\n",STR_dellist(sbp,0)) ;
    printf("buffer length = %d\n",STR_buflen(sbp)) ;
  }

  STR_free(sbp) ;
}

#endif
